export interface WebCallback {
    id: string;
    fun: (event: any) => void;
}
export declare class $WebSocket {
    private url;
    private protocols;
    private config;
    normalCloseCode: number;
    private reconnectableStatusCodes;
    private reconnectAttempts;
    private sendQueue;
    private onOpenCallbacks;
    private onMessageCallbacks;
    private onErrorCallbacks;
    private onCloseCallbacks;
    private readyStateConstants;
    private socket;
    private internalConnectionState;
    constructor(url: string, protocols?: Array<string>, config?: WebSocketConfig);
    /***************************
     * Connect/reconnect/close functions
     */
    connect(): void;
    reconnect(): void;
    close(force: boolean): void;
    /***********************
     * send websocket data.
     */
    send(data: any): Promise<{}>;
    fireQueue(): void;
    /**************************
     * Add callback functions
     */
    onOpen(cb: any): void;
    onClose(cb: any): void;
    onError(cb: any): void;
    onMessage(cb: any): void;
    /**************************
     * Remove callback functions
     */
    removeOnClose(key: any): void;
    removeOnError(key: any): void;
    removeOnOpen(key: any): void;
    removeOnMessage(key: any): void;
    /**************************
     * The callbacks are handled here!
     */
    onMessageHandler(message: MessageEvent): void;
    onCloseHandler(event: CloseEvent): void;
    onErrorHandler(event: any): void;
    onOpenHandler(event: Event): void;
    notifyCloseCallbacks(event: any): void;
    notifyErrorCallbacks(event: any): void;
    getReadyState(): number;
}
export interface WebSocketConfig {
    initialTimeout: number;
    maxTimeout: number;
    reconnectIfNotNormalClose: boolean;
    reconnectTimeout: number;
}
