"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var $WebSocket = (function () {
    function $WebSocket(url, protocols, config) {
        this.url = url;
        this.protocols = protocols;
        this.config = config;
        this.normalCloseCode = 1000;
        this.reconnectableStatusCodes = [4000];
        this.reconnectAttempts = 0;
        this.sendQueue = [];
        // Callbacks 
        this.onOpenCallbacks = {};
        this.onMessageCallbacks = {};
        this.onErrorCallbacks = {};
        this.onCloseCallbacks = {};
        // Constants 
        this.readyStateConstants = {
            'CONNECTING': 0,
            'OPEN': 1,
            'CLOSING': 2,
            'CLOSED': 3,
            'RECONNECT_ABORTED': 4
        };
        var match = new RegExp('wss?:\/\/').test(url);
        if (!match) {
            throw new Error('Invalid url provided');
        }
        this.config = config || { initialTimeout: 500, maxTimeout: 300000, reconnectIfNotNormalClose: false, reconnectTimeout: 10000 };
    }
    /***************************
     * Connect/reconnect/close functions
     */
    $WebSocket.prototype.connect = function () {
        var self = this;
        if (!this.socket || this.socket.readyState !== this.readyStateConstants.OPEN) {
            this.socket = new WebSocket(this.url);
            this.socket.onopen = function (ev) {
                self.onOpenHandler(ev);
            };
            this.socket.onmessage = function (ev) {
                self.onMessageHandler(ev);
            };
            this.socket.onclose = function (ev) {
                self.onCloseHandler(ev);
            };
            this.socket.onerror = function (ev) {
                self.onErrorHandler(ev);
            };
        }
    };
    $WebSocket.prototype.reconnect = function () {
        var _this = this;
        this.close(true);
        console.log('Reconnecting in ' + this.config.reconnectTimeout + ' ms.');
        setTimeout(function () { return _this.connect(); }, this.config.reconnectTimeout);
    };
    $WebSocket.prototype.close = function (force) {
        if (force || !this.socket.bufferedAmount) {
            this.socket.close();
        }
    };
    /***********************
     * send websocket data.
     */
    $WebSocket.prototype.send = function (data) {
        var self = this;
        return new Promise(function (resolve, reject) {
            if (self.socket.readyState === self.readyStateConstants.OPEN) {
                self.sendQueue.push({ message: data });
                self.fireQueue();
            }
            else {
                reject('Socket connection has been closed.');
            }
        });
    };
    $WebSocket.prototype.fireQueue = function () {
        while (this.sendQueue.length && this.socket.readyState === this.readyStateConstants.OPEN) {
            var data = this.sendQueue.shift();
            this.socket.send((typeof data.message === "string") ? data.message : JSON.stringify(data.message));
        }
    };
    /**************************
     * Add callback functions
     */
    $WebSocket.prototype.onOpen = function (cb) {
        this.onOpenCallbacks[cb.id] = cb.fun;
    };
    $WebSocket.prototype.onClose = function (cb) {
        this.onCloseCallbacks[cb.id] = cb.fun;
    };
    $WebSocket.prototype.onError = function (cb) {
        this.onErrorCallbacks[cb.id] = cb.fun;
    };
    $WebSocket.prototype.onMessage = function (cb) {
        this.onMessageCallbacks[cb.id] = cb.fun;
    };
    /**************************
     * Remove callback functions
     */
    $WebSocket.prototype.removeOnClose = function (key) {
        delete this.onCloseCallbacks[key];
    };
    $WebSocket.prototype.removeOnError = function (key) {
        delete this.onErrorCallbacks[key];
    };
    $WebSocket.prototype.removeOnOpen = function (key) {
        delete this.onOpenCallbacks[key];
    };
    $WebSocket.prototype.removeOnMessage = function (key) {
        delete this.onMessageCallbacks[key];
    };
    /**************************
     * The callbacks are handled here!
     */
    $WebSocket.prototype.onMessageHandler = function (message) {
        for (var key in this.onMessageCallbacks) {
            this.onMessageCallbacks[key].apply(this, [message]);
        }
    };
    $WebSocket.prototype.onCloseHandler = function (event) {
        this.notifyCloseCallbacks(event);
        if ((this.config.reconnectIfNotNormalClose && event.code !== this.normalCloseCode) || this.reconnectableStatusCodes.indexOf(event.code) > -1) {
            this.reconnect();
        }
    };
    $WebSocket.prototype.onErrorHandler = function (event) {
        this.notifyErrorCallbacks(event);
    };
    $WebSocket.prototype.onOpenHandler = function (event) {
        this.reconnectAttempts = 0;
        for (var key in this.onOpenCallbacks) {
            this.onOpenCallbacks[key](event);
        }
        this.fireQueue();
    };
    $WebSocket.prototype.notifyCloseCallbacks = function (event) {
        for (var key in this.onCloseCallbacks) {
            this.onCloseCallbacks[key](event);
        }
    };
    $WebSocket.prototype.notifyErrorCallbacks = function (event) {
        for (var key in this.onErrorCallbacks) {
            this.onErrorCallbacks[key](event);
        }
    };
    $WebSocket.prototype.getReadyState = function () {
        if (this.socket == null) {
            return -1;
        }
        return this.internalConnectionState || this.socket.readyState;
    };
    $WebSocket = __decorate([
        core_1.Injectable(), 
        __metadata('design:paramtypes', [String, Array, Object])
    ], $WebSocket);
    return $WebSocket;
}());
exports.$WebSocket = $WebSocket;
//# sourceMappingURL=angular2-websocket.js.map